/***************************************************************
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: do_stuff.c                                     *
*       Created: Thurday, Jan 16 02:59:22 1992                 *
*       Abstract:                                              *
*         This module contains all the do_xxxx routines called *
*         directly by the rteld process.                       *
*                                                              *
*       Input:                                                 *
*         As specified by each individual routine.             *
*                                                              *
*       Output:                                                *
*         As specified by each individual routine.             *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  20-may-92  Allow none to be specified as an option
                   for rteld in the server_hosts file.

   RKL  20-may-92  Make "service not offered on server" an
                   informational message.  If fatal, cannot do
		   virtual translation through rteld.

   RKL  16-jan-92  Created by splitting modules out from one big
                   file.  (process_pty.c)
*/

#include <udpinc.h>



/***************************************************************
*                                                              *
* DO_CLEANUP                                                   *
*                                                              *
*   This routine will get a directory listing of the           *
*   /usr/spool/rtel/files directory and step through it        *
*   looking for files of the form ttyXX.  If an appropriate    *
*   file is found, check to see if the process specified by    *
*   it's PID is running.  If not, remove the file, as it has   *
*   become stale.                                              *
*                                                              *
***************************************************************/

do_cleanup(users)
udp_data *users;
{
    FILE *lsfd, *psfd;
    char ls_buf[80], ps_buf[132], *ptr, server[80], service[80];
    int found, pid, pspid;

    
    sprintf(ls_buf, "ls %s", rtel_files);
    lsfd = popen(ls_buf, "r");
    
    while(fgets(ls_buf, sizeof(ls_buf), lsfd)){
	ptr = ls_buf;
	while(*ptr && (*ptr != '\r') && (*ptr != '\n'))ptr++;
	*ptr = 0;

	sprintf(users->fname,"%s/%s", rtel_files, ls_buf);
	if(parse_kill_file(users, &pid, service, server) == OK){

#ifdef BSD
	    psfd = popen("ps -ax", "r");
#else
	    psfd = popen("ps -e", "r");
#endif
	    found = FALSE;
	    fgets(ps_buf, sizeof(ps_buf), psfd);
	    
	    while(fgets(ps_buf, sizeof(ps_buf), psfd)){
		sscanf(ps_buf, "%d", &pspid);
		if(pspid == pid){
		    found = TRUE;
		}
	    }
	    pclose(psfd);

	    if(!found){
		printf("Removing stale file %s\r\n", users->fname);
		sprintf(ps_buf, "rm -f %s", users->fname);
		system(ps_buf);
	    }

	}
    }
    pclose(lsfd);
    return(OK);
}



/***************************************************************
*                                                              *
* DO_KILL                                                      *
*                                                              *
*   This routine will attempt to cleanly stop rteld processes  *
*   by executing the rteld kill files.                         *
*                                                              *
*   It will attempt to stop the rteld process associated with  *
*   the pseudo-tty device specified in users->ttydevm.  If the *
*   device name is "all" then all running rteld processes will *
*   be stopped.  Note that this command requires that the user *
*   have the appropriate privileges to stop the specified      *
*   daemon programs.                                           *
*                                                              *
***************************************************************/

do_kill(users)
udp_data *users;
{
    FILE *fkill;
    char ls_buf[80], *ptr, service[80], server[80];
    int pid;

    if (strcmp(users->ttydevm, "all")){
	ptr = users->ttydevm + 5;
	sprintf(users->fname,"%s/%s", rtel_files, ptr);

	if(parse_kill_file(users, &pid, service, server) < 0){
	    printf("Couldn't open kill file %s\r\n", users->fname);
	    return -1;
	}
	system(users->fname);
    }
    else{
	sprintf(ls_buf, "ls %s", rtel_files);
	fkill = popen(ls_buf, "r");

	while(fgets(ls_buf, 80, fkill)){
	    ptr = ls_buf;
	    while(*ptr && (*ptr != '\r') && (*ptr != '\n'))ptr++;
	    *ptr = 0;

	    sprintf(users->fname,"%s/%s", rtel_files, ls_buf);
	    if(parse_kill_file(users, &pid, service, server) == OK){
		system(users->fname);
	    }
	}
	pclose(fkill);
    }
    return(OK);
}



/***************************************************************
*                                                              *
* DO_REVIEW                                                    *
*                                                              *
*   This routine will show what rteld processes are currently  *
*   running.                                                   *
*                                                              *
*   It does this by using the popen system routine to get a    *
*   listing of all the kill files in the /usr/spool/rtel/files *
*   directory.  It then opens each of these files and formats  *
*   and prints the information they contain.                   *
*                                                              *
***************************************************************/

do_review(users)
udp_data *users;
{
    FILE *lsfile;
    char ls_buf[80], *ptr, service[80], server[80];
    int pid;

    printf("\r\n");
    printf("  Device          Service               Host              PID\r\n");
    printf("----------    ----------------    ----------------    ----------\r\n");
    sprintf(ls_buf, "ls %s", rtel_files);
    lsfile = popen(ls_buf, "r");

    while(fgets(ls_buf, sizeof(ls_buf), lsfile)){
	ptr = ls_buf;
	while(*ptr && (*ptr != '\r') && (*ptr != '\n'))ptr++;
	*ptr = 0;
	
	sprintf(users->fname,"%s/%s", rtel_files, ls_buf);
	if(parse_kill_file(users, &pid, service, server) == OK){
	    printf("/dev/%-5s    %-16s    %-16s    %d\r\n",
		   ls_buf, service, server, pid);
	}
    }
    printf("\r\n");
    pclose(lsfile);
}



/***************************************************************
*                                                              *
* DO_SETUP                                                     *
*                                                              *
*   This routine loop through the server_hosts file looking    *
*   for lines with a "/" in the first column.  It will then    *
*   create a rteld command to start each daemon process and    *
*   use the system routine to fork off the appropriate daemon. *
*                                                              *
***************************************************************/

do_setup(users)
udp_data *users;
{
    FILE *filep;
    char opts[80], buf[132];
    int rval;
    
    if ((filep = fopen(hostfile, "r")) == NULL) {
	printf ("Unable to open server_hosts file %s, errno--> %d\r\n", hostfile, errno);
	return(PROC_ERR);
    }
    
    while(fgets(buf, sizeof(buf), filep) != NULL) {
	if((buf[0] != '#') && (buf[0] == '/')){
	    if ((rval = sscanf (buf, "%s %s %s %s %[ -z]", 
				users->ttydevm, users->host, users->svcnm,
				users->passwd, opts)) < 5) {
		printf("Not enough parameters on line:\r\n%s\r\n", buf);
		continue;
	    }
	    
	    sprintf(buf,"%s/rteld -d%s ", rtel_dir, users->ttydevm);
	    system(buf);
	}
    }
    
    fclose (filep);
    return OK;
}



/***************************************************************
*                                                              *
* DO_TALK                                                      *
*                                                              *
*   This routine is used to actually talk to the server.       *
*                                                              *
***************************************************************/

do_talk(users)
udp_data *users;
{
    char     *arglist[10], args[10][40], *ptr1, *ptr2;
    unsigned short euid;
    int      argc, fd, master, optlen, stat;
    
    /* If the user specified that this is to become a permanent system daemon
       process, make sure that we are root.  */
    
    if(users->pty_flags & BE_DAEMON){
	if((euid = geteuid()) != 0){
	    printf ("Must be root to create a daemon process!!\r\n");
	    return(PROC_ERR);
	}
    }
    
    /* If a pseudo-tty name was not specified, find the first available pair.
       Otherwise, use the pseudo-device pair that the user specified.  Note
       that the open_pty routine automatically does this.  */
    
    if((stat = open_pty(users, &master)) < 0){
	return(PROC_ERR);
    }
    
    /* If the user did not specify a service name or host name, look up
       the entry in the server_hosts file to try to find a device entry
       with the associated information.

       If the device entry is found, construct a bogus argc/argv structure
       to pass to rteld_parse.  This is done to handle the arguments that
       the user may have put into the options field of the server_hosts file.
       Note that arguments may be specified both on the command line and in
       the server_hosts file.  If the options field in the server_hosts file
       contains "none", simply bypass this whole exercise.  */

    if(!users->svcnm[0] && !users->host[0]){
	if((stat = find_dev_name(users->ttydevs, users)) < 0) {
	    printf("Lookup failure in for %s in server_hosts file\r\n", users->ttydevs);
	    return(PROC_ERR);
	}

	if(strcmp(users->option, "none")){
	    argc = 1;
	    arglist[0] = args[0];
	    strcpy(args[0], "bogus");

	    ptr1 = users->option;
	    optlen = strlen(users->option);

	    while(ptr1 < (users->option + optlen)){
		ptr2 = ptr1;
		while(*ptr2 && (*ptr2 != ' '))ptr2++;
		*ptr2++ = 0;
		arglist[argc] = args[argc];
		strcpy(arglist[argc++], ptr1);
		ptr1 = ptr2;
	    }

	    if((stat = rteld_parse(argc, arglist, users)) == HELP){
		return(PROC_ERR);
	    }
	}
    }
    
    /* Check to see if the specified service is properly defined.  Note that
       this does not actually form a connection.  It just asks the target
       server if the service exists.  If it does not exist, simply issue a
       warning message and continue.  If we were to abort if the service
       is not found, rteld would never work with virtual translation
       services.  

       Note that if we are attempting a connection to a port number, there
       is no need to even look at the service.  */

    if(!users->hport[0]){
	if(connect_service(users, TEST_CONN) < 0){
	    printf("Service %s not offered on server %s.  Assuming a\r\n",
		   users->svcnm, users->host);
	    printf("virtual translation session wanted.  Processing continuing...\r\n");
	}
    }
    
    /* Check to see if a kill file already exists for this pseudo-tty.
       If so, we can't continue since someone there is already a daemon
       running or an incorrectly terminated daemon for this pty pair. */
    
    if(make_kill_file(users, CHECK_KILL) < 0) {
	printf("Kill file %s already exists\r\n", users->fname);
	return(PROC_ERR);
    }
    
    /* Tell the user what pseudo-tty we are running on.  From this point on,
       we will be running as either a system daemon process or as a child
       process of the original users's controlling terminal.  If running as
       a system daemon, call become_daemon.  If running as a child process,
       fork to return control to the user's terminal, close all open file
       descriptors and start handling the actual data stream.  */

    printf("%s\r\n",users->ttydevs);
    
    if (users->pty_flags & BE_DAEMON) {
	if (become_daemon(users) < 0){
	    return(PROC_ERR);
	}
    }
    else {
	if (fork()){
	    exit (0);
	}
	for (fd=0; fd < 10; fd++) close (fd);
    }
    process_pty(users);
}
