/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Richard K. Lyman                               *
*       Module: kill_files.c                                   *
*       Created: Wed Jan 22  18:34:00 1992                     *
*       Abstract:                                              *
*         These routines will handle kill file creation,       *
*         checking and deletion.                               *
*                                                              *
*       Input:                                                 *
*         ??                                                   *
*                                                              *
*       Output:                                                *
*         ??                                                   *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  12-may-92  Use kill -1 rather than kill -HUP since some
                   machines don't seem to allow kill -HUP in
		   the bourne shell.

   RKL  12-feb-92  Changed format of kill file to only delete
                   file when executed if successful in killing
		   the rteld process.

   RKL  22-jan-92  Split away into their own module.

 */

#include <udpinc.h>


parse_kill_file(users, pid, service, server)
udp_data *users;
int  *pid;
char *service;
char *server;
{
    FILE *kfile;
    char junkbuf[132];

    if ((kfile = fopen(users->fname, "r")) != NULL){
	fgets(junkbuf, sizeof(junkbuf), kfile);
	fscanf(kfile, "# service %s on host %s\n", service, server);
	fscanf(kfile, "if(kill -1 %d)\n", pid);
	fclose(kfile);
	return(OK);
    }
    return(PROC_ERR);
}






/*  This function is called to create a file that contains the
  information needed to kill off a rouge rteld process.
  
  If the kill file already exists, a message is displayed on
  the users terminal window so that he can either terminate
  the zombie process, or complete the proper cleanup procedure
  manually by executing rteld with the -k option.
  
  It the parameter check is specified, this routine will try
  to open the kill file.  If the open succeeds, the file will
  be closed and the routine will return a failure status.  */


make_kill_file(users, check)
udp_data *users;
int check;
{
    FILE *fkill;
    int fd, stat;
    
    /* Create the file name and try to open the kill file.  If 
       the open succeeds, return an error status since we should
       not be able to find the file at this point. */
    
    sprintf(users->fname, "%s%s",rtel_files, &users->ttydevs[4]); 
    
    if((fd = open(users->fname, O_RDONLY)) >= 0){
	errs(users, PTY_FILEXISTS, "process_pty:make_kill_file");
	close(fd);
	return(PROC_ERR);
    }

    /* If we are just checking to see if the file exists, return
       a good status, otherwise create the file and fill it with:

           #!/bin/sh
	   # service service_name on host host_name
	   if(kill -1 pid)
           then
	       rm /usr/spool/rtel/files/ttyXX
           fi
    
       This will kill with catchable signals the rteld process
       and remove all traces of the kill file. */

    if(check == CHECK_KILL){
	return(OK);
    }
    
    if((fd = creat(users->fname, KILL_MASK)) < 0){
	errs(users, PTY_BADKILIO, "process_pty:kilfilcrt");
	errs(users, errno, "process_pty:kilfilcrt");
	return(PROC_ERR);
    }
    
    fkill = fdopen(fd, "w");
    fprintf(fkill, "#!/bin/sh\n");
    if(strlen(users->svcnm)){
	fprintf(fkill, "# service %s on host %s\n", users->svcnm, users->host); 
    }
    else{
	fprintf(fkill, "# service %s on host %s\n", users->hport, users->host); 
    }	
    fprintf(fkill, "if(kill -1 %d)\n", getpid());
    fprintf(fkill, "then\n");
    fprintf(fkill, "    rm %s\n", users->fname);
    fprintf(fkill, "fi\n");
    
    fclose(fkill);
    close(fd);
    return 0;
}
