/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: messages.c                                     *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         The routines in this file handle issuing messages    *
*         to a console device and to the UDP daemon and filter *
*         program log files.                                   *
*                                                              *
*       Input:                                                 *
*         These routines typically receive a string to be      *
*         printed and a destination to print the string to.    *
*                                                              *
*       Output:                                                *
*         Messages are logged to log files or to the system    *
*         console depending on the severity of the message     *
*         and the destination specified for the message.       *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  16-jun-92  Removed unneeded routines check_swab and
                   routines dealing with queueing daemon.

   RKL  11-jun-92  Restructured error message printing a bit.

   RKL  19-jul-91  Added required parameters to err_handler
                   and errs routines.

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   RKL  28-feb-91  Removed several unneeded routines and
                   cleaned up.  Note that in several places
		   the constant 2 is used instead of the
		   symbolic value SEEK_END because several
		   machines have really screwey include
		   files.

   RKL  09-jan-91  Changed from RTAP to RTEL.

 */

#include <udpinc.h>


/* At this point, the errstring is the actual text of the
   error message and the loc_msg is the string of the form
   routine_name:unique_id */

err_handler(filename, errstring, loc_msg)
char *filename;
char *errstring;
char *loc_msg;
{
    int errfil;
    if ((errfil = open(filename, O_RDWR)) < 0) {
	creat(filename,PMOD);
	if ((errfil = open(filename, O_RDWR)) < 0) {
	    return PROC_ERR;
	}
	timestamp(filename, " New File");
    }
    lseek(errfil, 0, 2);
    write (errfil, loc_msg, strlen(loc_msg));
    write( errfil, "::", 2);
    write (errfil, errstring, strlen(errstring));
    write (errfil, "\n", 1);
    close (errfil);
    return OK;
}



int timestamp (filename, errstring)
char *filename;
char *errstring;
{
    int clock, errfil;
    char output[MAXLEN];
    
    if ((errfil = open(filename, O_RDWR)) >= 0) {
	lseek(errfil, 0, 2);
	
	sprintf(output, "--------------------------------------------------\r\n");
	write(errfil, output, strlen(output));
	
	clock = time(0);
	sprintf(output,"%s,  %s", errstring, ctime(&clock));
	write(errfil, output, strlen(output));
	close(errfil);
    }
}



wrcons (string)
char *string;
{
    FILE *console;
    console = fopen("/dev/console","w");
    write (fileno(console), CRLF, strlen(CRLF));
    write (fileno(console), string, strlen(string));
    fclose (console);
    return OK;
}



int open_jobstat_fil(users)
udp_data *users;
{
    int fil, rval;
    char buf[LOGFLSIZ],
    output[MAXLEN];
    
    if ((fil=open(jobstat,O_RDWR)) < 0) {
	sprintf(output,"RTEL job status file, Version %s", VERSION);
	err_handler(jobstat, output, "");
    }
    else{
	/* Attempt to read LOGFLSIZ bytes from the status file.  If
	   the read is successful, the file is to big and needs to
	   be recreated. */
	
	lseek (fil, 0, 0); /* go to beginning */
	if ((rval = read(fil, buf, LOGFLSIZ)) == LOGFLSIZ) {
	    close(fil);
	    unlink(jobstat);
	    sprintf(output,"RTEL job status file, Version %s", VERSION);
	    err_handler(jobstat, output, "");
	}
    }
    close (fil);
    return OK;
}
