/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: open_pty.c                                     *
*       Created: Thurs 9 May 10:17:51 1991                     *
*       Abstract:                                              *
*         This module will handle finding and opening pty      *
*         devices.                                             *
*                                                              *
*       Input:                                                 *
*         The ttydevs field will contain the pty name to open. *
*         If it is not specified, i.e. a null field,  simply   *
*         open the first pty found.                            *
*                                                              *
*       Output:                                                *
*         If a device is specified and could be opened, a fd   *
*         will be returned.  If the device could not be        *
*         opened, or a free pseudo-tty could not be found, an  *
*         error status will be returned.                       *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  08-jul-92  Added documentation.

*/

#include "udpinc.h" 

int open_pty(users, master_desc)
udp_data *users;
int  *master_desc;
{

    int  count, slave_desc;
    char c;
    char dev_name[MAXLEN];
    
    /* If the ttydevs field in the users structure already contains a name,
       attempt to open the master device of that pseudo-pty pair, otherwise
       loop through the psuedo terminals until we find an available one and
       open it.

       Note that these routines assume a device name of the form /dev/ptyXX
       for the pseudo-tty master device, and /dev/ttyXX as the slave. */

    if (users->ttydevs[0]) {
	strcpy(users->ttydevm, users->ttydevs);
	users->ttydevm[5] = 'p';
	if ((*master_desc = open(users->ttydevm, O_RDWR)) >= 0) {
	    return 0;
	}
	errs(users, errno, "open_pty:badopen");
	printf("PTY: Unable to open pseudo-tty pair %s.\r\n", users->ttydevm );
	return -1;
    }
    else{
	strcpy(dev_name, "/dev/pty  ");
	for(c = 'p'; c <= 's'; c++){
	    *(dev_name+8) = c;
	    for(count = 0; count <= 15; count++){
		*(dev_name+9) = count + ((count < 10) ? 48 : 97);
		if ((*master_desc = open(dev_name, O_RDWR)) >= 0) {
		    strcpy (users->ttydevm, dev_name);
		    *(dev_name+5) = 't';
		    strcpy (users->ttydevs, dev_name);
		    return 0;
		}
	    }
	}
	printf("PTY: Unable to allocate pseudo-tty pair.\r\n");
	return(PROC_ERR);
    }
}
