/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: udpinc.h                                       *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         Include file for the reverse telnet host software.   *
*                                                              *
*       Modification History:                                  *
*         10-oct-91  RKL  Corrected directories for Convergent *
*                         include files.                       *
*                                                              *
*         26-jul-91  RKL  Added changes for DYNIX systems.     *
*                                                              *
*         18-jul-91  RKL  Added changes for Stardent and ISC   *
*                         targets.                             *
*                                                              *
*         14-may-91  RKL  Merged changes for Phase II RTEL     *
*                                                              *
*         28-feb-91  RKL  Changed character declarations from  *
*                         unsigned char to char in udp data    *
*                         structure.                           *
*                                                              *
***************************************************************/

#define VERSION "V3.2"

/* Global include files. */

#include <sys/types.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <termio.h>

#include <sys/file.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <netinet/in.h>

/* OS specific include files. */

#ifdef AIX
#include <sys/select.h>
#endif

#ifdef ATT_WNG
#include <sys/in.h>
#endif

#ifdef CONV
#include <sys/in.h>
#endif

#ifdef DYNIX
#define _IOCTL_
#define SIGCLD SIGCHLD
FILE *popen();
#endif

#ifdef ISC
#include <net/errno.h>
#endif

#ifdef ST3
include <sys/ioctl.h>
#endif

#ifdef SVR4
#include <unistd.h>
#define getdtablesize()  ((int) sysconf(_SC_OPEN_MAX))
#endif

/* This is a bug in SCO's TCP/IP software.  It may be fixed some day... */

#ifdef SCO
#define getdtablesize() 60
#endif

/*********************** connection defines ***********************/
/* these values must not change since they are used by the server. */

#define QD              0x40
#define RECONNECT       0x80

#define RETRY           0x1000
#define ALLOW_QUE       0x2000
#define CONNECTED       0x4000

#define QUEUED          1
#define RTEL_PROT       2
#define MAKE_CONN       0x00
#define TEST_CONN       0x80

#define TYPE_END        0
#define TYPE_VERS       1
#define TYPE_CONN       2
#define TYPE_REPL       3
#define TYPE_SNMP_PORT  3
#define TYPE_SRCHSTNAM  4
#define TYPE_DSTHSTNAM  5
#define TYPE_DST_PORT   6

#define TCP             6
#define UDP             17

#define IAC             0xff
#define WILL            0xfb
#define WONT            0xfc
#define DO              0xfd
#define DONT            0xfe
#define TIMING_MARK     0x06

/**********  End of server (non-changable) parameters. ***********/

/* Major rteld functions */
#define SETUP         1
#define TALK          2
#define KILL          3
#define REVIEW        4
#define CLEAN         5
#define HELP          6
#define VERS          7

/* Miscellaneous defines */
#define BEENQUEUED    1
#define CHECK_KILL    1
#define CREATE_KILL   2
#define FALSE         0
#define LING_TIM      180
#define OK            0
#define PROC_ERR     -1
#define TRUE          1

/* Socket number to use for RTEL connections */
#define SOCK_RTEL     2000
#define SOCK_RTELIAC  3000

/* Backend program flag definitions (xxx->flags) */
#define OPT_BANNER    0x01
#define OPT_BINARY    0x02
#define OPT_EXPTAB    0x04
#define OPT_POST      0x08

/* PTY flag definitions (xxx->pty_flags) */
#define CONN_IMED     0x0001
#define BE_DAEMON     0x0002
#define QUEUE_PTY     0x0004

/********************** buffer and file defines **********************/

#define BUFFSIZE   512
#define LOGFLSIZ 20000
#define MAXLEN     132

#define CR         "\r"
#define LF         "\n"
#define CRLF       "\r\n"

#define PMOD       0776  /* anyone and anything can happen */
#define KILL_MASK  0755  /* allow read/execute for everyone */

#define RTEL      1      /* These constants define logfile locations. */
#define CONSOLE   2
#define TERMINAL  3

/* reject/disconnect codes received from the tserver */

#define LAT_BASE        10000           /* when referencing, subtract base */
#define LAT_REJNORMAL   LAT_BASE+0      /* normal node close */
#define LAT_REJCIROOR   LAT_BASE+1      /* circuit timer out of range */
#define LAT_REJSHUTDOWN LAT_BASE+2      /* system shutting down */
#define LAT_REJCOMPROTO LAT_BASE+3      /* communication protocol error */
#define LAT_REJINVSVC   LAT_BASE+4      /* invalid service class */
#define LAT_REJINSRES   LAT_BASE+5      /* insufficient node resources */
#define LAT_REJINUSE    LAT_BASE+6      /* service in use */
#define LAT_REJNOSUCHSV LAT_BASE+7      /* no such service */
#define LAT_REJSVDIS    LAT_BASE+8      /* service disabled */
#define LAT_REJSVPORT   LAT_BASE+9      /* service not on requested port */
#define LAT_REJPORT     LAT_BASE+10     /* object port not known */
#define LAT_REJINVPASS  LAT_BASE+11     /* invalid password */
#define LAT_REJQUEENT   LAT_BASE+12     /* queue request not in found */
#define LAT_REJIMMACC   LAT_BASE+13     /* immediate access denied */
#define LAT_REJACC      LAT_BASE+14     /* access denied */
#define LAT_REJCORSOL   LAT_BASE+15     /* corrupt solicit */

/* rtel file processing errors */

#define RTEL_BASE        11000
#define RTEL_HSTNTFND    RTEL_BASE+1      /* host missing in /etc/host file */
#define RTEL_NOCONN      RTEL_BASE+2      /* tcp connection attempt failed */
#define RTEL_BADSOCK     RTEL_BASE+3      /* got a bad socket from net-sftwr */
#define RTEL_SELEXCEPT   RTEL_BASE+4      /* exception during select */
#define RTEL_BADWR       RTEL_BASE+5      /* bad port/pid write to queue file*/
#define RTEL_FINNED      RTEL_BASE+6      /* got finned on connect request */
#define RTEL_BADDATA     RTEL_BASE+7      /* got confusing data from tserver */
#define RTEL_NODAMN      RTEL_BASE+8      /* daemon files missing */
#define RTEL_BADRD       RTEL_BASE+9      /* bad read from daemon file */
#define RTEL_BADHSTOPN   RTEL_BASE+10     /* bad open on server_hosts file */
#define RTEL_BADOPN      RTEL_BASE+11     /* bad open on status files */
#define RTEL_TIMSTMP     RTEL_BASE+12     /* couldn't timestamp status files */
#define RTEL_NOFILE      RTEL_BASE+13     /* no filename w/print-reqiest */
#define RTEL_NOTFINISHED RTEL_BASE+14     /* job finished with errors */
#define RTEL_BUSY        RTEL_BASE+19     /* service requested is busy */
#define RTEL_CNTOPNJOB   RTEL_BASE+20     /* could not open job_status file */
#define RTEL_Q_HSTNTFND  RTEL_BASE+21     /* host missing in server_hosts */
#define RTEL_Q_DEVNTFND  RTEL_BASE+22     /* tty/dev missing in server_hosts */
#define RTEL_BADSCANF    RTEL_BASE+23     /* bad scanf in server_hosts file */
#define RTEL_ABORT       RTEL_BASE+24     /* read/write damon, child aborted */
#define RTEL_NOBANNER    RTEL_BASE+25     /* unable to open banner file */
#define RTEL_QFULL       RTEL_BASE+26     /* request denied, tserver q is full */

/* pty file processing errors */

#define PTY_BASE        13000
#define PTY_BADKILFIL    PTY_BASE+1       /* Unable to create kill file */
#define PTY_FILEXISTS    PTY_BASE+2       /* kill file already exists */
#define PTY_MPTYBUSY     PTY_BASE+3       /* error opening master pty */
#define PTY_BADKILIO     PTY_BASE+4       /* can't open kill file */



/********************** RTEL file name defines  **********************/

#define rtel_dir     "/usr/spool/rtel"
#define rtel_files   "/usr/spool/rtel/files"
#define jobstat      "/usr/spool/rtel/job_status"
#define joboldstat   "/usr/spool/rtel/job_ostat"
#define hostfile     "/usr/spool/rtel/server_hosts"


/*************** Structures to use if DEBUG flag is defined ************/
#ifdef DEBUG
#define dbgout(s)        wrcons(s)
#define dprintf(x,y,z)   sprintf(x,y,z)
char dbgbuf[512];
#else
#define dbgout(s)
#define dprintf(x,y,z)
#endif

/********************** Miscellaneous Structures  **********************/


struct timerval {
  long    tv_sec;    /* seconds */
  long    tv_usec;   /* and microseconds, 10ths of seconds */
};

typedef struct{
  struct sockaddr_in
                 temp_sockdata,     /* for getting information */
                 tcp_sockdata,      /* TCP (data) session network data */
                 udp_sockdata;      /* UDP (queueing) session network data */

  FILE *         fdp;               /* file to be printed */

  unsigned char  usr_buf[BUFFSIZE]; /* buffer for read/write */

  char           ermssg[10][MAXLEN], /* 10 text error messages */
                 host[MAXLEN],      /* name of target tserver */
                 svcnm[MAXLEN],     /* name of service to use */
                 passwd[MAXLEN],    /* service password */
                 hnode[MAXLEN],     /* node name for service of form svcnm@host */
                 hport[MAXLEN],     /* port number/name on the server */
                 usernam[MAXLEN],   /* name of the user who requested job */
                 fname[MAXLEN],     /* name of file to print */
                 option[MAXLEN],    /* options field specified for rteld */
                 ttydevm[12],       /* name of the master tty device device */
                 ttydevs[12];       /* name of the slave tty device */

  int            next,              /* pointer of where to put next error */
                 copies,            /* number of copies to print */
                 flags,             /* for setting up connection */
                 pty_flags,         /* flags for pseudo-terminal connections */
                 erarray[10];       /* errors to log */

  unsigned short tcp_fd,            /* TCP file descriptor for data transfer */
                 tcp_socket,        /* TCP socket number for data stream */
                 uniq_id,           /* original outgoing port number for queueing */
                 udp_fd,            /* UDP file descriptor for queueing information */
                 udp_socket;        /* UDP socket number for queueing */

} udp_data;
